@@ -46,7 +46,7 @@ class KeyValueStore :
 		_, _, data = KeyValueStore._client.get((self._namespace, self._set, key))
 		self._cache[key] = (time() + self._local_TTL, data['data'])
 
-		return data['data']
+		return copy(data['data'])
 
 
 	def get(self: 'KeyValueStore', key: str) -> Any :
@@ -66,15 +66,25 @@ class KeyValueStore :
 
 		if remote_keys :
 			data: List[Tuple[Any]] = KeyValueStore._client.get_many(list(map(lambda k : (self._namespace, self._set, k), remote_keys)))
+			data_map: Dict[str, Any] = { }
+
+			exp: float = time() + self._local_TTL
+			for datum in data :
+				key: str = datum[0][2]
+
+				# filter on the metadata, since it will always be populated
+				if datum[1] :
+					value: Any = datum[2]['data']
+					data_map[key] = copy(value)
+					self._cache[key] = (exp, value)
+
+				else :
+					data_map[key] = None
 
 			return {
+				**data_map,
 				**{
-					datum[0][2]: datum[2]['data']
-					# filter on the metadata, since it will always be populated
-					for datum in filter(lambda x : x[1], data)
-				},
-				**{
-					key: self._cache[key][1]
+					key: copy(self._cache[key][1])
 					for key in keys - remote_keys
 				},
 			}
@@ -95,3 +105,30 @@ class KeyValueStore :
 		async with self._get_many_lock :
 			__clear_cache__(self._cache, time)
 			return self._get_many(keys)
+
+
+	def remove(self: 'KeyValueStore', key: str) -> None :
+		if key in self._cache :
+			del self._cache[key]
+
+		self._client.remove(
+			(self._namespace, self._set, key),
+			policy={
+				'max_retries': 3,
+			},
+		)
+
+
+	def exists(self: 'KeyValueStore', key: str) -> bool :
+		try :
+			_, meta = self._client.exists(
+				(self._namespace, self._set, key),
+				policy={
+					'max_retries': 3,
+				},
+			)
+			# check the metadata, since it will always be populated
+			return meta != None
+
+		except aerospike.exceptions.RecordNotFound :
+			return False
