@@ -1,5 +1,5 @@
+from kh_common.server.middleware import CustomHeaderMiddleware, HeadersToSet
 from starlette.middleware.trustedhost import TrustedHostMiddleware
-from kh_common.server.middleware import CustomHeaderMiddleware
 from kh_common.server.middleware.auth import KhAuthMiddleware
 from kh_common.server.middleware.cors import KhCorsMiddleware
 from kh_common.exceptions.base_error import BaseError
@@ -73,6 +73,7 @@ def ServerApp(
 	allowed_protocols = ['http', 'https'] if environment.is_local() else ['https']
 
 	if custom_headers :
+		exposed_headers = list(exposed_headers) + list(HeadersToSet.keys())
 		app.middleware('http')(CustomHeaderMiddleware)
 
 	if cors :
@@ -80,9 +81,9 @@ def ServerApp(
 			KhCorsMiddleware,
 			allowed_origins = set(allowed_origins),
 			allowed_protocols = set(allowed_protocols),
-			allowed_headers = allowed_headers,
-			allowed_methods = allowed_methods,
-			exposed_headers = exposed_headers,
+			allowed_headers = list(allowed_headers),
+			allowed_methods = list(allowed_methods),
+			exposed_headers = list(exposed_headers),
 			max_age = max_age,
 		)
 
